require "rss/rss"

module RSS

	module RSS09
		NSPOOL = {}
		ELEMENTS = []
	end

	class Rss < Element

		include RSS09

		[
			["channel", nil],
		].each do |tag, occurs|
			install_model(tag, occurs)
		end

		%w(channel).each do |x|
			install_has_child_element(x)
		end

		attr_accessor :rss_version, :version, :encoding, :standalone
		
		def initialize(rss_version, version=nil, encoding=nil, standalone=nil)
			@rss_version = rss_version
			@version = version || '1.0'
			@encoding = encoding
			@standalone = standalone
		end

		def output_encoding=(enc)
			@output_encoding = enc
			self.converter = Converter.new(@output_encoding, @encoding)
		end
		
		def items
			if @channel
				@channel.item(0..-1)
			else
				[]
			end
		end

		def to_s(convert=true)
			rv = <<-EOR
#{xmldecl}
<rss version="#{@rss_version}"
	#{ns_declaration}>
#{channel_element(false)}
#{other_element(false)}
</rss>
EOR
      rv = @converter.convert(rv) if convert and @converter
      rv
		end

		private
		def xmldecl
			rv = "<?xml version='#{@version}'"
			if @output_encoding or @encoding
				rv << " encoding='#{@output_encoding or @encoding}'" 
			end
			rv << " standalone='#{@standalone}'" if @standalone
			rv << '?>'
			rv
		end

		def ns_declaration
			NSPOOL.collect do |prefix, uri|
				prefix = ":#{prefix}" unless prefix.empty?
				%Q|xmlns#{prefix}="#{uri}"| 
			end.join("\n")
		end
		
		def children
			[@channel]
		end

		class Channel < Element

			include RSS09
			
			%w(title link description language copyright managingEditor
			   webMaster rating pubDate lastBuildDate docs category
			   generator ttl skipDays skipHours).each do |x|
				install_text_element(x)
			end

			%w(image textInput cloud).each do |x|
				install_has_child_element(x)
			end
			
			%w(item).each do |x|
				install_has_children_element(x)
			end
			
			def initialize()
				@item = []
			end

			def to_s(convert=true)
				rv = <<-EOT
<channel>
	#{title_element(false)}
	#{link_element(false)}
	#{description_element(false)}
	#{language_element(false)}
	#{copyright_element(false)}
	#{managingEditor_element(false)}
	#{webMaster_element(false)}
	#{rating_element(false)}
	#{pubDate_element(false)}
	#{lastBuildDate_element(false)}
	#{docs_element(false)}
	#{category_element(false)}
	#{generator_element(false)}
	#{ttl_element(false)}
	#{skipDays_element(false)}
	#{skipHours_element(false)}
	#{image_element(false)}
	#{item_element(false)}
	#{textinput_element(false)}
  #{other_element(false)}
</channel>
EOT
	      rv = @converter.convert(rv) if convert and @converter
  	    rv
			end

	    private
			def children
				[@image, @textInput, @cloud]
			end

			class Image < Element

				include RSS09
				
				%w(url title link width height description).each do |x|
					install_text_element(x)
				end

				def to_s(convert=true)
					rv = <<-EOT
<image>
	#{url_element(false)}
	#{title_element(false)}
	#{link_element(false)}
	#{width_element(false)}
	#{height_element(false)}
	#{description_element(false)}
  #{other_element(false)}
</image>
EOT
	     		rv = @converter.convert(rv) if convert and @converter
	  	    rv
				end

			end
			
			class Cloud < Element

				include RSS09
				
				[
					["domain", nil, false],
					["port", nil, false],
					["path", nil, false],
					["registerProcedure", nil, false],
					["protocol", nil ,false],
				].each do |name, uri, required|
					install_get_attribute(name, uri, required)
				end

				def to_s(convert=true)
					rv = <<-EOT
<cloud
	domain="#{@domain}"
	port="#{@port}"
	path="#{@path}"
	registerProcedure="#{@registerProcedure}"
	protocol="#{@protocol}"
/>
EOT
	     		rv = @converter.convert(rv) if convert and @converter
	  	    rv
				end

			end
			
			class Item < Element
				
				include RSS09

				%w(title link description author category comments
				   enclosure guid pubDate source).each do |x|
					install_text_element(x)
				end

				def to_s(convert=true)
					rv = <<-EOT
<item>
	#{title_element(false)}
	#{link_element(false)}
	#{description_element(false)}
	#{author_element(false)}
	#{category_element(false)}
	#{comments_element(false)}
	#{enclosure_element(false)}
	#{guid_element(false)}
	#{pudDate_element(false)}
	#{source_element(false)}
  #{other_element(false)}
</item>
EOT
	     		rv = @converter.convert(rv) if convert and @converter
	  	    rv
				end

			end
			
			class TextInput < Element
				
				include RSS09

				%w(title description name link).each do |x|
					install_text_element(x)
				end

				def to_s(convert=true)
					rv = <<-EOT
<textInput>
	#{title_element(false)}
	#{description_element(false)}
	#{name_element(false)}
	#{link_element(false)}
  #{other_element(false)}
</textInput>
EOT
	     		rv = @converter.convert(rv) if convert and @converter
	  	    rv
				end

			end
			
		end
		
	end

	if const_defined?(:BaseListener)
		RSS09::ELEMENTS.each do |x|
			BaseListener.install_get_text_element(x, nil, "#{x}=")
		end
	end

	if const_defined?(:ListenerMixin)
		module ListenerMixin
			private
			def start_rss(tag_name, prefix, attrs, ns)
				check_ns(tag_name, prefix, ns, nil)

				@rss = Rss.new(attrs['version'], @version, @encoding, @standalone)
				@last_element = @rss
				@proc_stack.push Proc.new { |text, tags|
					@rss.stream_validate(tags) if @do_validate
				}
			end
					
		end
	end

end
